import re

def clean_string(target_str):
    """ Remove ISO control characters and trim input string. Returns None if cleaned string is empty.
    Args:
        target_str (st): string to be cleaned.
    Returns:
        str: cleaned input string.
    """
    if target_str is None:
        return None
    else:
        string_clean = re.sub(r'[\x00-\x1F]+', '', target_str).strip()
        if string_clean == '':
            return None
        else:
            return string_clean

def empty_string_to_null(target_str):
    """ Check if input string is empty, and return null if so (otherwise return input string).
    Args:
        target_str (str): string to check for emptiness.
    Returns:
        str: null if input string is empty else input string.
    """
    if target_str is None:
        return None
    elif re.sub(r'[\x00-\x1F]+', '', target_str).strip() == '':
        return None
    else:
        return target_str


import pandas as pd
from uuid import uuid4
from pyspark.sql.functions import pandas_udf, PandasUDFType, udf
from pyspark.sql.types import DoubleType, StringType
from .general_udfs_base import clean_string, empty_string_to_null


@pandas_udf(StringType(), PandasUDFType.SCALAR)
def pd_clean_string(target_col):
    """ Apply `clean_string` over Spark Column.
    Args:
        target_col (Spark Column): containing strings to clean.
    Returns:
        Spark Column (StringType): cleaned version of input strings.
    """
    return pd.Series(target_col.apply(lambda x: clean_string(x)))

@pandas_udf(StringType(), PandasUDFType.SCALAR)
def pd_empty_string_to_null(target_col):
    """ Apply `empty_string_to_null` to Spark Column.
    Args:
        target_col (Spark Column): containing strings to convert empties --> nulls
    Returns:
        Spark Column: where empty strings replaced with nulls.
    """
    return pd.Series(target_col.apply(lambda x: empty_string_to_null(x)))

def pd_map_string(broadcast_map):
    """ Apply `map dict` to Spark Column, and return prior value if not containe 
        in map.

    Usage example:
        df.withColumn("new_col", pd_map_string(broadcast_variable)(F.col("old_col"))).show()

    Args:
        target_col (Spark Column): containing strings to convert with map
    Returns:
        Spark Column: where strings replaced with map.
    """
    def f(x):
        if x in broadcast_map.value:
            return broadcast_map.value.get(x)
        else:
            return x
    return udf(f)


import pandas as pd

def pd_read_map(map_csv_path, delimiter=";"):
    """ Read mapping csv from hdfs as dict, consist of source (string to replace)
        and destination (string for replace with) column
        
        from:    
        +-----------+--------------+
        | source    | destination  |
        +-----------+--------------+
        | perempuan | wanita       |
        | laki-laki | pria         |
        +-----------+--------------+
        
        to:
        {"perempuan":"wanita", "laki-laki":"pria"}

        Args:
            map_csv_path (String): map csv hdfs path
            delimiter (String): delimiter of csv file. default ";"

        Return:
            Mapping dictionary
    """
    mapx = spark.read.option("delimiter", delimiter)\
                     .option("header",True)
                     .csv(map_csv_path)
    mapx = pd.Series(
        mapx.toPandas().source.values,
        index=mapx.toPandas().destination
    ).to_dict()

    return spark.sparkContext.broadcast(mapx)   
